require 'spec_helper'

describe Rack::UsefulProcline do
  it 'has a version number' do
    expect(Rack::UsefulProcline::VERSION).not_to be nil
  end
  
  describe 'on initialization' do
    subject { Rack::UsefulProcline.new(rack_app) }
  end
end

describe Rack::UsefulProcline::Config do
  describe 'default config' do
    subject do
      Rack::UsefulProcline.new(rack_app).config
    end
    
    it { expect(subject.opts[:request_uuid_key]).to eql('action_dispatch.request_id') }
    it { expect(subject.opts[:show_request_uuid]).to be true }
    it { expect(subject.opts[:show_request_method]).to be true }
    it { expect(subject.opts[:show_request_path]).to be true }
  end
  
  describe 'modified config' do
    before(:each) do
      Rack::UsefulProcline.configure do |config|
        config.opts[:request_uuid_key] = 'my_request_uuid_key'
        config.opts[:show_request_uuid] = false
        config.opts[:show_request_method] = false
        config.opts[:show_request_path] = false
      end
    end
    
    subject do
      Rack::UsefulProcline.new(rack_app).config
    end
    
    it { expect(subject.opts[:request_uuid_key]).to eql('my_request_uuid_key') }
    it { expect(subject.opts[:show_request_uuid]).to be false }
    it { expect(subject.opts[:show_request_method]).to be false }
    it { expect(subject.opts[:show_request_path]).to be false }
  end

end

describe Rack::UsefulProcline do
  subject { Rack::UsefulProcline.new(rack_app) }
  
  describe '#call' do
    subject do
      Rack::UsefulProcline.new(rack_app).call(rack_env)
    end
    
    it { expect(subject[0]).to eql("200") }
    it { expect(subject[1]).to eql({'Content-Type' => 'text/html'}) }
    it { expect(subject[2]).to eql(['A Rack app!']) }
  end
  
  describe '#sanitize_utf8' do
    # not sure what to do here
  end
  
  describe '#set_procline' do
    # or here
  end
  
  describe '#request_uuid' do
    it 'should retrieve the truncated request uuid' do
      env = {'action_dispatch.request_id' => 'my_request_uuid'}
      subject.instance_variable_set(:@request, Rack::Request.new(env))
      expect(subject.request_uuid.length).to eql(8)
    end
  end
  
  describe '#procline_before_request' do
    subject do 
      s = Rack::UsefulProcline.new(rack_app)
      s.instance_variable_set(:@request, Rack::Request.new(rack_env))
      s.procline_before_request("")
    end
    
    describe "with default config (everything true)" do
      it { expect(subject).to include("GET") }
      it { expect(subject).to include("/path/to/things") }
      it { expect(subject).to include("12345678") }
    end
    
    describe "with show_request_method, show_request_uuid and show_request_path turned off" do
      subject do
        Rack::UsefulProcline.configure do |config| 
          config.opts[:show_request_method] = false
          config.opts[:show_request_path] = false
          config.opts[:show_request_uuid] = false
        end
        Rack::UsefulProcline.new(rack_app).procline_before_request("")
      end
      
      it do 
        expect(subject).not_to include("GET")
        expect(subject).not_to include("/path/to/things")
        expect(subject).not_to include("12345678")
      end
    end
    
    describe "with custom request_uuid_key" do
      subject do
        key = "my_request_uuid_key"
        Rack::UsefulProcline.configure { |config| config.opts[:request_uuid_key] = key }
        s = Rack::UsefulProcline.new(rack_app)
        env = rack_env(request_uuid_key: key)
        s.instance_variable_set(:@request, Rack::Request.new(env))
        s.procline_before_request("")
      end
      
      it { expect(subject).to include("12345678") }
    end
  end
  
  describe '#procline_after_request' do
    before(:each) do
      def create_subject(opts = {})
        opts[:rack_app] ||= rack_app
        opts[:rack_env] ||= rack_env
        s = Rack::UsefulProcline.new(opts[:rack_app])
        s.instance_variable_set(:@request, Rack::Request.new(opts[:rack_env]))
        s.procline_after_request("", "200")
      end
    end
    
    describe "with default config (everything true)" do
      subject { create_subject }
      
      it do
        expect(subject).not_to include("GET")
        expect(subject).not_to include("/path/to/things")
        expect(subject).to include("12345678")
        expect(subject).to include("last status: 200.")
        expect(subject).to include("Waiting for req.")
      end
    end
    
    describe "with show_request_method, show_request_uuid and show_request_path turned off" do
      subject do
        Rack::UsefulProcline.configure do |config| 
          config.opts[:show_request_method] = false
          config.opts[:show_request_path] = false
          config.opts[:show_request_uuid] = false
        end
        create_subject
      end
      
      it do 
        expect(subject).not_to include("GET")
        expect(subject).not_to include("/path/to/things")
        expect(subject).not_to include("12345678")
        expect(subject).to include("last status: 200.")
        expect(subject).to include("Waiting for req.")
      end
    end
    
    describe "with custom request_uuid_key" do
      subject do
        key = 'my_request_uuid_key'
        Rack::UsefulProcline.configure { |config| config.opts[:request_uuid_key] = key }
        env = rack_env(request_uuid_key: key)
        create_subject(rack_env: env)
      end
      
      it { expect(subject).to include("12345678") }
    end
  end
end
